import React, { useState } from 'react'
import { useDrop } from 'react-dnd'
import { is, fromJS } from 'immutable'
import update from 'immutability-helper'

import Utils from '@/utils/utils.js'
import Card from './card'
import './index.scss'

const Container = ({list, type, handleList, handleMenu, deleteMenu, profileMenu, changeBtnStyle, dropButton, doubleClickCard }) => {
  const [cards, setCards] = useState(list)
  const moveCard = (id, atIndex) => {
    const { card, index } = findCard(id)
    const _cards = update(cards, { $splice: [[index, 1], [atIndex, 0, card]] })
    handleList(_cards)
  }

  if (!is(fromJS(cards), fromJS(list))) {
    setCards(list)
  }
  
  const findCard = id => {
    const card = cards.filter(c => `${c.uuid}` === id)[0]
    return {
      card,
      index: cards.indexOf(card),
    }
  }

  const doubleClickBtn = id => {
    const { card } = findCard(id)
    doubleClickCard(card)
  }

  const editCard = id => {
    const { card } = findCard(id)
    handleMenu(card)
  }
  
  const profileCard = id => {
    const { card } = findCard(id)
    profileMenu(card)
  }

  const changeStyle = id => {
    const { card } = findCard(id)
    changeBtnStyle(card)
  }

  const delCard = id => {
    const { card } = findCard(id)
    deleteMenu(card)
  }

  const copyCard = id => {
    const { card } = findCard(id)

    let copycard = fromJS(card).toJS()

    copycard.origin = false
    copycard.copyType = 'action'
    copycard.focus = true

    let _val = fromJS(copycard).toJS()

    if (_val.control) {
      _val.control = ''
      
      delete _val.controlField
      delete _val.controlVal
    }

    copycard.uuid = Utils.getuuid()
    copycard.originCard = card

    try {
      _val = window.btoa(window.encodeURIComponent(JSON.stringify(_val)))
    } catch (e) {
      console.warn('Stringify Failure')
      _val = ''
    }

    if (_val) {
      let oInput = document.createElement('input')
      oInput.value = _val
      document.body.appendChild(oInput)
      oInput.select()
      document.execCommand('Copy')
      document.body.removeChild(oInput)
    }

    const { index: overIndex } = findCard(id)

    const _cards = update(cards, { $splice: [[overIndex + 1, 0, copycard]] })

    handleList(_cards, copycard)
  }

  const [, drop] = useDrop({
    accept: 'action',
    drop(item) {
      const { index } = findCard(item.id)
      if (index > -1) return
      dropButton(item.id)
    }
  })

  return (
    <div ref={drop} className="ant-row">
      {cards.map(card => (
        <Card
          id={card.uuid}
          key={card.uuid}
          card={card}
          type={type}
          moveCard={moveCard}
          copyCard={copyCard}
          editCard={editCard}
          delCard={delCard}
          findCard={findCard}
          changeStyle={changeStyle}
          profileCard={profileCard}
          doubleClickCard={doubleClickBtn}
        />
      ))}
    </div>
  )
}
export default Container
